Attribute VB_Name = "MyCommands"
'' Created by Lee Ambrosius (lee_ambrosius@hyperpics.com), HyperPics all rights reserved.
'' Visit HyperPics on the web at: http://www.hyperpics.com
'' Date: 12/29/03

'' Version 1.0 - Works with 2004
'' CustomDWGProps is a command line extension of the Summary Info API or the DWGPROPS command.
'' It allows you to list all fields and also add/remove user defind DWG Properties.  Along
'' with supporting the custom Properties it has full support for the several Standard Properties
'' as well.

'' To use wrapper copy and paste into text file and save it with the extension .LSP
'' Make sure to remove the word "Rem" at the beginning of each line

'' Begin wrapper code (Copy below until End wrapper code)
Rem ;; Example of AutoLISP wrapper for command line support
Rem ;; Make sure CustomDWGProps.dvb is in the AutoCAD Support Paths
Rem (defun c:CustomDWGProps ()
Rem   ;; Load is ignored if file is already loaded
Rem   ;; (vl-vbaload "<ProjectFile>.dvb")
Rem   (vl-vbaload "CustomDWGProps.dvb")
Rem
Rem   ;; (vl-vbarun "<Module>"."<FunctionName>")
Rem   (vl-vbarun "MyCommands.CustomDwgProps")(princ)
Rem )
'' End wrapper code

'' Custom message handler to avoid unecessry "Command: " at the command line
Private Sub printMsg(msg As String)
  Dim vNoMutt As Variant
  vNoMutt = ThisDrawing.GetVariable("NoMutt")
  ThisDrawing.SetVariable "NoMutt", 0
  ThisDrawing.Utility.Prompt vbLf + msg
  ThisDrawing.SetVariable "NoMutt", vNoMutt
End Sub

Public Sub CustomDWGProps()
  Dim vNoMutt As Variant
  vNoMutt = ThisDrawing.GetVariable("NoMutt")
  ThisDrawing.SetVariable "NoMutt", 1
  
  AppActivate ThisDrawing.Application.Caption
  
  Dim acSumInfo As AcadSummaryInfo

  Set acSumInfo = ThisDrawing.SummaryInfo

  On Error Resume Next

  Dim strKWord As String
  ThisDrawing.Utility.InitializeUserInput 0, "Standard Custom ?"
  printMsg "DWG Properties [Standard/Custom/?]: "
  strKWord = ThisDrawing.Utility.GetKeyword() ''vbLf + "DWG Properties [Standard/Custom/?]: ")

  Dim strFieldName As String, strFieldValue As String, strFieldValueTest As String

  ''Test return value
  If strKWord = "Standard" Then
    Err.Clear
    ThisDrawing.Utility.InitializeUserInput 0, "Title Author Subject Hyperlinkbase Comments Keywords Lastsavedby Revisionnumber"
    printMsg "Standard DWG Properties [Title/Subject/Author/Keywords/Comments/Hyperlinkbase/Lastsavedby/Revisionnumber]: "
    strKWord = ThisDrawing.Utility.GetKeyword() ''vbLf + "Standard DWG Properties [Title/Subject/Author/Keywords/Comments/Hyperlinkbase/Lastsavedby/Revisionnumber]: ")

    If strKWord <> "" Then
      '' Get the Field Value from the user
      printMsg "Enter Field Value: "
      strFieldValue = ThisDrawing.Utility.GetString(True) '', vbLf + "Enter Field Value: ")

      If strFieldValue <> "" Then
        With acSumInfo
          Select Case strKWord
            Case "Title"
              .Title = strFieldValue
            Case "Author"
              .Author = strFieldValue
            Case "Subject"
              .Subject = strFieldValue
            Case "Hyperlinkbase"
              .HyperlinkBase = strFieldValue
            Case "Comments"
              .Comments = strFieldValue
            Case "Lastsavedby"
              .LastSavedBy = strFieldValue
            Case "Revisionnumber"
              .RevisionNumber = strFieldValue
          End Select
        End With
      End If
    End If
  ElseIf strKWord = "Custom" Then
    ThisDrawing.Utility.InitializeUserInput 0, "Add Modify Remove"
    printMsg "Custom DWG Properties [Add/Modify/Remove]: "
    strKWord = ThisDrawing.Utility.GetKeyword() ''vbLf + "Custom DWG Properties [Add/Modify/Remove]: ")

    '' Get the Field Name from the user
    printMsg "Enter Field Name: "
    strFieldName = ThisDrawing.Utility.GetString(True) '', vbLf + "Enter Field Name: ")
    
    '' It's easy enough to combine Add and Modify code into one function
    If strKWord = "Add" Or strKWord = "Modify" Then
      Err.Clear
      '' Get the Field Value from the user
      printMsg "Enter Field Value: "
      strFieldValue = ThisDrawing.Utility.GetString(True) '', vbLf + "Enter Field Value: ")
      
      '' Check to see if the Custom Property already exists
      acSumInfo.GetCustomByKey strFieldName, strFieldValueTest
      
      '' Check for Err and if Number is greater than 0 it doesn't exist yet
      If Err.Number <> 0 Then
        acSumInfo.AddCustomInfo strFieldName, strFieldValue
      Else
        acSumInfo.SetCustomByKey strFieldName, strFieldValue
      End If
      
      Err.Clear
    ElseIf strKWord = "Remove" Then '' Remove entry
      acSumInfo.RemoveCustomByKey strFieldName
    End If
  ElseIf strKWord = "?" Then
    '' Print title
    With acSumInfo
      printMsg vbLf + "Standard Properties "
      printMsg " Title: " + .Title

      '' Print Subject
      printMsg " Subject: " + .Subject
      '' Print Author
      printMsg " Author: " + .Author
      '' Print Keywords
      printMsg " Keywords: " + .Keywords
      '' Print Comments
      printMsg " Comments: " + .Comments
      '' Print Hyperlink Base
      printMsg " Hyperlink Base: " + .HyperlinkBase
      '' Print Last Saved By
      printMsg " Last Saved By: " + .LastSavedBy
      '' Print Revision Number
      printMsg " Revision Number: " + .RevisionNumber

      '' Get the number of Custom properties
      Dim nCustomProps As Integer
      nCustomProps = .NumCustomInfo

      Dim strKey As String, strValue As String

      '' Print Custom Properties if there are any
      If nCustomProps > 0 Then
        printMsg vbLf + "Custom Properties"

        For I = 0 To nCustomProps - 1
          .GetCustomByIndex I, strKey, strValue
          '' Print Custom Properties if there is any
          printMsg " " + strKey + ": " + strValue
        Next I
        
        printMsg vbLf
      Else
        '' No Custom Properties exist
        printMsg vbLf + "No Custom Properties" + vbLf
      End If
    End With
  End If

  ThisDrawing.SetVariable "NoMutt", vNoMutt
  Set acSumInfo = Nothing
End Sub
